@@ -316,6 +316,11 @@ class DatascopeScenario(Scenario, abstract=True):
             id=self.id,
             dataset=self.dataset,
             pipeline=self.pipeline,
+            model=self.model,
+            providers=self.providers,
+            mc_timeout=self.mc_timeout,
+            mc_tolerance=self.mc_tolerance,
+            nn_k=self.nn_k,
             method=self.method,
             utility=self.utility,
             iteration=self.iteration,
